<?php
/**
 * Unit tests for mod/vpl/locallib.php.
 *
 * @version		$Id: testvplfilegroup.php,v 1.3 2010/07/02 10:53:35 juanca Exp $
 * @package		vpl. Unit tests for mod/vpl/locallib.php
 * @copyright	Copyright (C) 2009 Juan Carlos Rodríguez-del-Pino. All rights reserved.
 * @license		GNU/GPL, see LICENSE.txt or http://www.gnu.org/licenses/gpl-2.0.html
 * @author		Juan Carlos Rodriguez-del-Pino jcrodriguez@dis.ulpgc.es
 **/

if (!defined('MOODLE_INTERNAL')) {
	die('Direct access to this script is forbidden.');
}

require_once($CFG->dirroot.'/mod/vpl/filegroup.class.php');

class vpl_filegroup_test extends UnitTestCase {
	var $fg1;
	var $fg2;
	var $fg3;
	var $fg4;
	var $td1;
	var $td2;
	var $td3;
	var $td4;
	
	
	function setUp(){
		global $CFG;
		$tempdir=$CFG->dataroot.'/temp';
		$this->td1 = tempnam($tempdir,'tst');
		mkdir($this->td1.'dir');
		$this->td2 = tempnam($tempdir,'tst');
		mkdir($this->td2.'dir');
		$this->td3 = tempnam($tempdir,'tst');
		mkdir($this->td3.'dir');
		unlink($this->td1);
		unlink($this->td2);
		unlink($this->td3);
		$this->td1.='dir';
		$this->td2.='dir';
		$this->td3.='dir';
		$this->fg1 = new file_group_process($this->td1.'/f1.lst',$this->td1,10);
		$this->fg2 = new file_group_process($this->td2.'/f2.lst',$this->td2,5,2);
		$this->fg3 = new file_group_process($this->td3.'/f3.lst',$this->td3,3,3);
	}
	
	function tearDown(){
		$this->assertEqual($this->fg1->get_maxnumfiles(),10);
		$this->assertEqual($this->fg2->get_maxnumfiles(),5);
		$this->assertEqual($this->fg3->get_maxnumfiles(),3);
		$this->assertEqual($this->fg1->get_numstaticfiles(),0);
		$this->assertEqual($this->fg2->get_numstaticfiles(),2);
		$this->assertEqual($this->fg3->get_numstaticfiles(),3);
		$this->assertTrue(vpl_delete_dir($this->td1));
		$this->assertTrue(vpl_delete_dir($this->td2));
		$this->assertTrue(vpl_delete_dir($this->td3));
	}

	function test_construct(){
	/**
	 * Constructor
	 *
	 * @param string $filelistname
	 * @param string $dir
	 * @param int $maxnumfiles
	 * @param int $numstaticfiles
	 */
	//function __construct($filelistname,$dir,$maxnumfiles=10000,$numstaticfiles=0){
		//get_maxnumfiles
		$this->assertEqual($this->fg1->get_maxnumfiles(),10);
		$this->assertEqual($this->fg2->get_maxnumfiles(),5);
		$this->assertEqual($this->fg3->get_maxnumfiles(),3);
		//get_numstaticfiles
		$this->assertEqual($this->fg1->get_numstaticfiles(),0);
		$this->assertEqual($this->fg2->get_numstaticfiles(),2);
		$this->assertEqual($this->fg3->get_numstaticfiles(),3);
	}

	function check_fg($fg, $names, $dataset, $limit){
		$list = $fg->getFileList();
		$this->assertEqual(count($list),$limit);
		for($j = 0 ; $j < $limit; $j++){
			$this->assertEqual($fg->getFileData($names[$j]),$dataset[$j]);
			$this->assertEqual($fg->getFileData($j),$dataset[$j]);
			$this->assertEqual($list[$j],$names[$j]);
		}
	}
	function test_addFile(){
		
	/**
	 * Add a new file to the group/Modify the data file
	 *
	 * @param string $filename
	 * @param string $data
	 * @return bool (added==true)
	 */
	//function addFile($filename,$data=null){
		$dataset = array('data f1', "data\nf2", '','data f4', 'f5', '', "f7\nline2\nline3\n", "f7\nline2\nline3\n");
		$names = array('f1', 'f2', 'f3','f4.bin', 'f5', 'f6.txt', 'f7', 'f8');
		//Adding files
		for($i = 0 ; $i <count($dataset); $i++){
			$this->assertTrue($this->fg1->addFile($names[$i],$dataset[$i]));
			$this->check_fg($this->fg1,$names, $dataset, $i+1);
		}
		//Check rename
		$this->assertTrue($this->fg1->renameFile(0,'f1mod'));
		$names[0] = 'f1mod';
		$this->check_fg($this->fg1,$names, $dataset, count($dataset));
		$this->assertTrue($this->fg1->renameFile(3,'f4mod'));
		$names[3] = 'f4mod';
		$this->check_fg($this->fg1,$names, $dataset, count($dataset));
		$this->assertTrue($this->fg1->renameFile(count($dataset)-1,'last'));
		$names[count($dataset)-1] = 'last';
		$this->check_fg($this->fg1,$names, $dataset, count($dataset));
		$this->assertFalse($this->fg1->renameFile(count($dataset)+1,'nothere'));
		$this->check_fg($this->fg1,$names, $dataset, count($dataset));
		$this->assertFalse($this->fg1->renameFile(-1,'nothere'));
		$this->check_fg($this->fg1,$names, $dataset, count($dataset));
		$this->assertFalse($this->fg1->renameFile(5,'f5'));
		$this->check_fg($this->fg1,$names, $dataset, count($dataset));
	}

	function test_deleteFile(){
	/**
	 * Delete a file from groupfile
	 *
	 * @param int $num file position
	 * @return bool
	 */
	//function deleteFile($num){
		//Delete file from 0 to num of files-1
		$dataset = array('data f1', "data\nf2", '','data f4', 'f5');
		$names = array('f1', 'f2', 'f3','f4.bin', 'f5');
		for($i = 0 ; $i <5; $i++){
			$this->assertTrue($this->fg2->addFile($names[$i],$dataset[$i]));
		}
		for($i = 0 ; $i <5; $i++){
			if($i<2){
				$this->assertFalse($this->fg2->deleteFile($i));
				$this->check_fg($this->fg2,$names, $dataset, count($dataset));
			}else{
				$this->assertTrue($this->fg2->deleteFile(2));
				$this->assertFalse(file_exists($this->td2.$names[2]));
				unset($names[2]);
				$names = array_values($names);
				unset($dataset[2]);
				$dataset = array_values($dataset);
				$this->check_fg($this->fg2,$names, $dataset, count($dataset));
			}
		}
		//Delete file from num of files-1 to 0
		$dataset = array('data f1', "data\nf2", '','data f4', 'f5');
		$names = array('f1', 'f2', 'f3','f4.bin', 'f5');
		for($i = 0 ; $i <5; $i++){
			$this->assertTrue($this->fg2->addFile($names[$i],$dataset[$i]));
		}
		for($i = 4 ; $i >=0; $i--){
			if($i<2){
				$this->assertFalse($this->fg2->deleteFile($i));
				$this->check_fg($this->fg2,$names, $dataset, count($dataset));
			}else{
				$this->assertTrue($this->fg2->deleteFile($i));
				unset($names[$i]);
				$names = array_values($names);
				unset($dataset[$i]);
				$dataset = array_values($dataset);
				$this->check_fg($this->fg2,$names, $dataset, count($dataset));
			}
		}
		//Delete file random
		$dataset = array('data f1', "data\nf2", '','data f4', 'f5');
		$names = array('f1', 'f2', 'f3','f4.bin', 'f5');
		$elements = array(1,3,0,2,1,4,5,2);
		for($i = 0 ; $i <5; $i++){
			$this->assertTrue($this->fg2->addFile($names[$i],$dataset[$i]));
		}
		foreach($elements as $pos){
			if($pos<2 || $pos >= count($dataset)){
				$this->assertFalse($this->fg2->deleteFile($pos));
				$this->check_fg($this->fg2,$names, $dataset, count($dataset));
			}else{
				$this->assertTrue($this->fg2->deleteFile($pos));
				unset($names[$pos]);
				$names = array_values($names);
				unset($dataset[$pos]);
				$dataset = array_values($dataset);
				$this->check_fg($this->fg2,$names, $dataset, count($dataset));
			}
		}
		//Check to delete a 3/3 fg
		//Delete file from 0 to num of files-1
		$dataset = array('data f1', "data\nf2", '');
		$names = array('f1', 'f2', 'f3');
		for($i = 0 ; $i <3; $i++){
			$this->assertTrue($this->fg3->addFile($names[$i],$dataset[$i]));
		}
		for($i = 0 ; $i <3; $i++){
			$this->assertFalse($this->fg3->deleteFile($i));
			$this->check_fg($this->fg3,$names, $dataset, count($dataset));
		}
		for($i = 2 ; $i >= 0; $i--){
			$this->assertFalse($this->fg3->deleteFile($i));
			$this->check_fg($this->fg3,$names, $dataset, count($dataset));
		}
		
	}

	function test_renameFile(){
	/**
	 * Rename a file
	 *
	 * @param int $num
	 * @param string $filename new filename
	 * @return bool (renamed==true)
	 */
	//function renameFile($num,$filename){
		//rename 5/3
		$dataset = array('data f1', "data\nf2", '','data f4', 'f5');
		$names = array('f1', 'f2', 'f3','f4.bin', 'f5');
		for($i = 0 ; $i <5; $i++){
			$this->assertTrue($this->fg2->addFile($names[$i],$dataset[$i]));
		}
		for($i = 0 ; $i <5; $i++){
			if($i<2){
				$this->assertFalse($this->fg2->renameFile($i,'rename'.$names[$i]));
				$this->assertFalse($this->fg2->renameFile($i,$names[($i+1)%5]));
				$this->check_fg($this->fg2,$names, $dataset, count($dataset));
			}else{
				$this->assertFalse($this->fg2->renameFile($i,$names[($i+1)%5]));
				$this->assertTrue($this->fg2->renameFile($i,'rename'.$names[$i]));
				$names[$i] = 'rename'.$names[$i];
				$this->check_fg($this->fg2,$names, $dataset, count($dataset));
			}
		}
		//Check to rename a 3/3 fg
		$dataset = array('data f1', "data\nf2", '');
		$names = array('f1', 'f2', 'f3');
		for($i = 0 ; $i <3; $i++){
			$this->assertTrue($this->fg3->addFile($names[$i],$dataset[$i]));
		}
		for($i = 0 ; $i <3; $i++){
			$this->assertFalse($this->fg3->renameFile($i,'rename'.$names[$i]));
			$this->assertFalse($this->fg3->renameFile($i,$names[($i+1)%3]));
		}
	}

	function test_is_populated(){
		$dataset = array('data f1', "data\nf2", '','data f4', 'f5', '', "f7\nline2\nline3\n", "f7\nline2\nline3\n");
		$names = array('f1', 'f2', 'f3','f4.bin', 'f5', 'f6.txt', 'f7', 'f8');
		//Adding files from 0
		for($i = 0 ; $i <count($names); $i++){
			$this->assertTrue($this->fg1->addFile($names[$i]));
			$this->assertFalse($this->fg1->is_populated());
		}
		for($i = 0 ; $i <count($names); $i++){
			$this->assertTrue($this->fg1->addFile($names[$i],$dataset[$i]));
			$this->assertTrue($this->fg1->is_populated());
		}
		for($i = 0 ; $i <count($names); $i++){
			$this->assertTrue($this->fg1->is_populated());
			$this->assertTrue($this->fg1->deleteFile(0));
		}
		//Adding files from count($dataset)-1
		for($i = 0 ; $i <count($names); $i++){
			$this->assertTrue($this->fg1->addFile($names[$i]));
			$this->assertFalse($this->fg1->is_populated());
		}
		$pos=count($dataset)-1;
		for($i = 0 ; $i <count($dataset); $i++){
			$this->assertTrue($this->fg1->addFile($names[$pos--],$dataset[$i]));
			$this->assertTrue($this->fg1->is_populated());
		}
		for($i = 0 ; $i <count($names); $i++){
			$this->assertTrue($this->fg1->is_populated());
			$this->assertTrue($this->fg1->deleteFile(0));
		}
		//Adding files in ramdom order
		$pos=count($dataset)/2;
		for($i = 0 ; $i <count($dataset)/2; $i++){
			$this->assertTrue($this->fg1->addFile($pos-$i,$dataset[$i]));
			$this->assertTrue($this->fg1->is_populated());
			$this->assertTrue($this->fg1->addFile($pos+$i,$dataset[$i]));
			$this->assertTrue($this->fg1->is_populated());
		}
	}
}

?>
