<?php
/**
 * Unit tests for mod/vpl/locallib.php.
 *
 * @version		$Id: testvpllocallib.php,v 1.3 2010-07-02 10:55:24 juanca Exp $
 * @package		vpl. Unit tests for mod/vpl/locallib.php
 * @copyright	Copyright (C) 2009 Juan Carlos Rodríguez-del-Pino. All rights reserved.
 * @license		GNU/GPL, see LICENSE.txt or http://www.gnu.org/licenses/gpl-2.0.html
 * @author		Juan Carlos Rodriguez-del-Pino jcrodriguez@dis.ulpgc.es
 **/

if (!defined('MOODLE_INTERNAL')) {
	die('Direct access to this script is forbidden.');
}

require_once($CFG->dirroot.'/mod/vpl/locallib.php');

class vpl_locallib_test extends UnitTestCase {

	function test_vpl_fopen(){
		/**
		 * Open/create a file and its dir
		 *
		 * @param $filename path to file
		 * @return file descriptor
		 **/
		//vpl_fopen($filename)
		$mes = 'Errors creating data to test vpl_fopen';
		global $CFG;
		$tempdir=$CFG->dataroot.'/temp';
		$dirname = tempnam ($tempdir,'tst');
		@unlink($dirname);
		if($this->assertTrue(mkdir($dirname),$mes)){
			$fname=$dirname.'/test.txt';
			$fd=vpl_fopen($fname);
			fwrite($fd,'data');
			fclose($fd);
			$this->assertEqual(file_get_contents($fname),'data');
			$fd=vpl_fopen($fname);
			fwrite($fd,'other data');
			fclose($fd);
			$this->assertEqual(file_get_contents($fname),'other data');
			$fname=$dirname.'/dir1/dir2/test.txt';
			$fd=vpl_fopen($fname);
			fwrite($fd,'data');
			fclose($fd);
			$this->assertEqual(file_get_contents($fname),'data');
			$fd=vpl_fopen($fname);
			fwrite($fd,'d');
			fclose($fd);
			$this->assertEqual(file_get_contents($fname),'d');
			vpl_delete_dir($dirname);
		}
	}

	function test_vpl_delete_dir(){

		/**
		 * Recursively delete a directory
		 *
		 * @return bool All delete
		 **/
		//vpl_delete_dir($dirname)
		$mes = 'Errors creating data to test vpl_delete_dir';
		global $CFG;
		$tempdir=$CFG->dataroot.'/temp';
		$fname = tempnam ($tempdir,'tst');
		@unlink($fname);
		if($this->assertTrue(mkdir($fname),$mes)){
			vpl_delete_dir($fname);
			$this->assertFalse(file_exists($fname));
		}
		if($this->assertTrue(mkdir($fname),$mes)){
			$this->assertTrue(mkdir($fname.'/a'),$mes);
			$this->assertTrue(touch($fname.'/af'),$mes);
			$this->assertTrue(mkdir($fname.'/b'),$mes);
			$this->assertTrue(mkdir($fname.'/c'),$mes);
			$this->assertTrue(touch($fname.'/Ac'),$mes);
			$this->assertTrue(mkdir($fname.'/b/a'),$mes);
			$this->assertTrue(mkdir($fname.'/c/a'),$mes);
			$this->assertTrue(touch($fname.'/c/af'),$mes);
			$this->assertTrue(touch($fname.'/c/a/f'),$mes);
			$this->assertTrue(mkdir($fname.'/c/b'),$mes);
			$this->assertTrue(mkdir($fname.'/b/a/a'),$mes);
			$this->assertTrue(mkdir($fname.'/b/a/b'),$mes);
			$this->assertTrue(mkdir($fname.'/b/a/c'),$mes);
			$this->assertTrue(touch($fname.'/b/a/c/8'),$mes);
			$this->assertTrue(touch($fname.'/b/a/c/7'),$mes);
			$this->assertTrue(touch($fname.'/b/a/c/6'),$mes);
			$this->assertTrue(mkdir($fname.'/b/a/d'),$mes);
			vpl_delete_dir($fname);
			$this->assertFalse(file_exists($fname));
		}
	}

	function test_vpl_read_list_from_file(){
		/**
		 * get parsed lines of a file
		 *
		 * @param $filename string
		 * @return array of lines of the file
		 **/
		//function vpl_read_list_from_file($filename){
		global $CFG;
		$tempdir=$CFG->dataroot.'/temp';
		$fname = tempnam ($tempdir,'tst');
		$data = array(
		array(),
		array('one line'),
		array('one line', 'two lines'),
		array('1 line', '2 lines', '3 lines'),
		array('one line', 'two lines', 'three lines', 'four lines')
		);
		$res = array('',
		'one line',
		"one line\ntwo lines",
		"1 line\n2 lines\n3 lines",
		"one line\ntwo lines\nthree lines\nfour lines");
		for($i=0; $i< count($data); $i++){
			file_put_contents($fname,$res[$i]);
			$this->assertEqual(vpl_read_list_from_file($fname),$data[$i]);
		}
		@unlink($fname);
	}

	function test_vpl_write_list_to_file(){
		/**
		 * Save an array in a file
		 * @param $filename string
		 **/
		//function vpl_write_list_to_file($filename, $list){
		global $CFG;
		$tempdir=$CFG->dataroot.'/temp';
		$fname = tempnam ($tempdir,'tst');
		$data = array(
		array(),
		array('','one line', 'two lines'),
		array('1 line', '', '2 lines', '3 lines'),
		array('one line', 'two lines', 'three lines', 'four lines')
		);
		$res = array('',
		"one line\ntwo lines",
		"1 line\n2 lines\n3 lines",
		"one line\ntwo lines\nthree lines\nfour lines");
		for($i=0; $i< count($data); $i++){
			vpl_write_list_to_file($fname,$data[$i]);
			$this->assertEqual(file_get_contents($fname),$res[$i]);
		}
		@unlink($fname);
	}

	function test_vpl_get_lang(){
		/**
		 * Get lang code
		 * @parm $bash_adapt true adapt lang to bash LANG (default false)
		 * @return string
		 */
		//function vpl_get_lang($bash_adapt=false){
	}

	function test_vpl_abs_href(){
		/**
		 * generate URL to page with params
		 * @param $page string page from wwwroot
		 * @param $var1 string var1 name optional
		 * @param $value1 string value of var1 optional
		 * @param $var2 string var2 name optional
		 * @param $value2 string value of var2 optional
		 * @param ...
		 **/
		//function vpl_abs_href(){
		global $CFG;
		$url = 'view.php';
		$rurl =  $CFG->wwwroot.$url;
		$parms  = array('',''     ,'parm'             , '.p_ / =');
		$values = array('','value','value with spaces', '.p_ / = other chars');
		$res    = array('=','=value','parm=value+with+spaces', '.p_+%2F+%3D=.p_+%2F+%3D+other+chars');
		$this->assertEqual(vpl_abs_href($url),$rurl);
		for($i=0; $i<count($parms); $i++){
			$j = ($i+1) % count($parms);
			$k = ($j+1) % count($parms);
			$this->assertEqual(vpl_abs_href($url,$parms[$i],$values[$i]),$rurl.'?'.$res[$i]);
			$this->assertEqual(vpl_abs_href($url,$parms[$i],$values[$i],$parms[$j],$values[$j]),
			$rurl.'?'.$res[$i].'&amp;'.$res[$j]);
			$this->assertEqual(vpl_abs_href($url,$parms[$i],$values[$i],$parms[$j],$values[$j],$parms[$k],$values[$k]),
			$rurl.'?'.$res[$i].'&amp;'.$res[$j].'&amp;'.$res[$k]);
		}	}

		function test_vpl_mod_href(){

			/**
			 * generate URL to page with params
			 * @param $page string page from wwwroot/mod/vpl/
			 * @param $var1 string var1 name optional
			 * @param $value1 string value of var1 optional
			 * @param $var2 string var2 name optional
			 * @param $value2 string value of var2 optional
			 * @param ...
			 **/
			//function vpl_mod_href(){
			global $CFG;
			$url = 'view.php';
			$rurl =  $CFG->wwwroot.'/mod/vpl/'.$url;
			$parms  = array('',''     ,'parm'             , '.p_ / =');
			$values = array('','value','value with spaces', '.p_ / = other chars');
			$res    = array('=','=value','parm=value+with+spaces', '.p_+%2F+%3D=.p_+%2F+%3D+other+chars');
			$this->assertEqual(vpl_mod_href($url),$rurl);
			for($i=0; $i<count($parms); $i++){
				$j = ($i+1) % count($parms);
				$k = ($j+1) % count($parms);
				$this->assertEqual(vpl_mod_href($url,$parms[$i],$values[$i]),$rurl.'?'.$res[$i]);
				$this->assertEqual(vpl_mod_href($url,$parms[$i],$values[$i],$parms[$j],$values[$j]),
				$rurl.'?'.$res[$i].'&amp;'.$res[$j]);
				$this->assertEqual(vpl_mod_href($url,$parms[$i],$values[$i],$parms[$j],$values[$j],$parms[$k],$values[$k]),
				$rurl.'?'.$res[$i].'&amp;'.$res[$j].'&amp;'.$res[$k]);
			}
		}

		function test_vpl_rel_url(){
			/**
			 * generate URL relative page with params
			 * @param $page string page relative
			 * @param $var1 string var1 name optional
			 * @param $value1 string value of var1 optional
			 * @param $var2 string var2 name optional
			 * @param $value2 string value of var2 optional
			 * @param ...
			 **/
			//function vpl_rel_url(){
			$url = 'http://someplace.com/info';
			$parms  = array('',''     ,'parm'             , '.p_ / =');
			$values = array('','value','value with spaces', '.p_ / = other chars');
			$res    = array('=','=value','parm=value+with+spaces', '.p_+%2F+%3D=.p_+%2F+%3D+other+chars');
			$this->assertEqual(vpl_rel_url(''),'');
			$this->assertEqual(vpl_rel_url($url),$url);
			for($i=0; $i<count($parms); $i++){
				$j = ($i+1) % count($parms);
				$k = ($j+1) % count($parms);
				$this->assertEqual(vpl_rel_url('',$parms[$i],$values[$i]),'?'.$res[$i]);
				$this->assertEqual(vpl_rel_url($url,$parms[$i],$values[$i]),$url.'?'.$res[$i]);
				$this->assertEqual(vpl_rel_url('',$parms[$i],$values[$i],$parms[$j],$values[$j]),
			'?'.$res[$i].'&amp;'.$res[$j]);
				$this->assertEqual(vpl_rel_url($url,$parms[$i],$values[$i],$parms[$j],$values[$j]),
				$url.'?'.$res[$i].'&amp;'.$res[$j]);
				$this->assertEqual(vpl_rel_url('',$parms[$i],$values[$i],$parms[$j],$values[$j],$parms[$k],$values[$k]),
			'?'.$res[$i].'&amp;'.$res[$j].'&amp;'.$res[$k]);
				$this->assertEqual(vpl_rel_url($url,$parms[$i],$values[$i],$parms[$j],$values[$j],$parms[$k],$values[$k]),
				$url.'?'.$res[$i].'&amp;'.$res[$j].'&amp;'.$res[$k]);
			}
		}

		function test_vpl_url_add_param(){

			/**
			 * Add a parm to a url
			 * @param $url string
			 * @param $parm string name
			 * @param $value string value of parm
			 **/
			//function vpl_url_add_param($url,$parm,$value){
			$url1 = 'http://someplace.com/info';
			$url2 = 'http://someplace.com/info?parm1=value1';
			$url3 = 'http://someplace.com/info?parm1=value1&amp;parm2=value2';
			$parms  = array('',''     ,'parm'             , '.p_ / =');
			$values = array('','value','value with spaces', '.p_ / = other chars');
			$res    = array('=','=value','parm=value+with+spaces', '.p_+%2F+%3D=.p_+%2F+%3D+other+chars');
			for($i=0; $i<count($parms); $i++){
				$this->assertEqual(vpl_url_add_param('',$parms[$i],$values[$i]),'?'.$res[$i]);
				$this->assertEqual(vpl_url_add_param($url1,$parms[$i],$values[$i]),$url1.'?'.$res[$i]);
				$this->assertEqual(vpl_url_add_param($url2,$parms[$i],$values[$i]),$url2.'&amp;'.$res[$i]);
				$this->assertEqual(vpl_url_add_param($url3,$parms[$i],$values[$i]),$url3.'&amp;'.$res[$i]);
			}
		}

		function test_vpl_redirect(){

			/**
			 * Print a message and redirect
			 * @param $message string to be print
			 * @param $link URL to redirect to
			 * @param $wait int time to wait in seconds
			 * @return void
			 */
			//function vpl_redirect($message,$link,$wait=4){
		}

		function test_vpl_inmediate_redirect(){

			/**
			 * Inmediate redirect
			 * @param $link URL to redirect to
			 * @return void
			 */
			//function vpl_inmediate_redirect($url){
		}

		function test_vpl_include_jsfile(){

			//function vpl_include_jsfile($file,$defer=false){
		}

		function test_vpl_include_js(){
			//function vpl_include_js($jscript,$defer=false){
		}

		function test_vpl_js_alert(){

			/**
			 * Popup message box to show text
			 * @param $text text to show. It use s() to sanitize text
			 * @param $print
			 * @return void
			 */
			//function vpl_js_alert($text,$print=true){
		}

		function test_vpl_get_select_time(){
			//function vpl_get_select_time($limit=null){
			$secminutes = 60;
			$times = array(null,0,10, 60, 120, 60*70) ;
			foreach ($times as $time){
				$data = vpl_get_select_time($time);
				if($this->assertIsA($data,'array') && $this->assertTrue(count($data) < 100)){
					foreach($data as $value => $valuetext){
						$this->assertTrue($value >= 0);
						if($time !== null){
							$this->assertTrue($value <= $time);
						}
						$this->assertIsA($valuetext,'string');
						if($value >= 60){
							$this->assertEqual($value % 60, 0);
						}
					}
				}
			}
		}

		function test_vpl_conv_size_to_string(){
			$this->assertEqual(vpl_conv_size_to_string(1024),'   1 KB');
			$this->assertEqual(vpl_conv_size_to_string(100*1024),' 100 KB');
			$this->assertEqual(vpl_conv_size_to_string(1024*999),' 999 KB');
			$this->assertEqual(vpl_conv_size_to_string(1024*1024),'   1 MB');
			$this->assertEqual(vpl_conv_size_to_string(1024*1024*2.1),'   2 MB');
			$this->assertEqual(vpl_conv_size_to_string(1024*1024*100),' 100 MB');
			$this->assertEqual(vpl_conv_size_to_string(1024*1024*999.9),' 999 MB');
			$this->assertEqual(vpl_conv_size_to_string(1024*1024*1000),'   1 GB');
			$this->assertEqual(vpl_conv_size_to_string(1024*1024*1100),'   1 GB');
			$this->assertEqual(vpl_conv_size_to_string(1024*1024*1000*999.9),' 999 GB');
			$this->assertEqual(vpl_conv_size_to_string(1024*1024*1000000),'   1 TB');
			$this->assertEqual(vpl_conv_size_to_string(1024*1024*1000000*1.1),'   1 TB');
		}
		
		function test_vpl_get_select_sizes(){
			/**
			 * Return un array with the format [size in bytes]=> size in text text
			 * The first element is [0] => select
			 * @param $limit the limit of values generates
			 * @return array
			 */
			//function vpl_get_select_sizes($limit=null){
			$sizes = array(0,20000, 100000, 10000000, 4000000000) ;
			foreach ($sizes as $size){
				$data = vpl_get_select_sizes($size);
				if($this->assertIsA($data,'array') && $this->assertTrue(count($data) < 30)){
					foreach($data as $value => $valuetext){
						$this->assertTrue($value >= 0);
						$this->assertTrue($value <= $size);
						$this->assertIsA($valuetext,'string');
					}
				}
			}
		}

		function test_vpl_detect_newline(){
			/**
			 * @param string $data
			 * @return string newline separator "\r\n", "\n\r", "\n"
			 */
			//function vpl_detect_newline(&$data){
			//Empty string
			$data = "";
			$temp = $data;
			$this->assertEqual(vpl_detect_newline($temp),"\n");
			$this->assertEqual($data,$temp);
			//New line in "\r\n"
			$data = "\r\n";
			$temp = $data;
			$this->assertEqual(vpl_detect_newline($temp),"\r\n");
			$this->assertEqual($data,$temp);
			//New line in "n"
			$data = "\n";
			$temp = $data;
			$this->assertEqual(vpl_detect_newline($temp),"\n");
			$this->assertEqual($data,$temp);
			//Unix
			$data = "check unix new line\nlast line";
			$temp = $data;
			$this->assertEqual(vpl_detect_newline($temp),"\n");
			$this->assertEqual($data,$temp);
			//Windows
			$data = "check Windows new line\r\nlast line";
			$temp = $data;
			$this->assertEqual(vpl_detect_newline($temp),"\r\n");
			$this->assertEqual($data,$temp);
			//Mac
			$data = "check Mac new line\rlast line";
			$temp = $data;
			$this->assertEqual(vpl_detect_newline($temp),"\r");
			$this->assertEqual($data,$temp);
		}

		function test_vpl_display_grade() {
			//function vpl_display_grade($grade,$scaleid) {
			//Need access to scale db table
		}

}

?>
